#ifndef GAME_H
#define GAME_H

#define SUIT_DIAMONDS 0
#define SUIT_CLUBS    1
#define SUIT_HEARTS   2
#define SUIT_SPADES   3

typedef struct {
	unsigned char suit;
	unsigned char rank;
	unsigned char face_up;
	} Card;

typedef struct CardListStruct CardList;
struct CardListStruct {
	Card card;
	CardList *next;
	};

enum UndoType { UNDO_MOVE, UNDO_REVEAL, UNDO_FINISH, UNDO_DEAL, };
typedef struct UndoStruct Undo;
struct UndoStruct {
	enum UndoType type;
	int fromSlot;
	int toSlot;
	int cardNum;
	Undo *next;
};

#define NUM_CARDS_IN_STOCK 104
#define NUM_TABLEAU_SLOTS  10

extern int numSuitsToUse;
extern Card deck[NUM_CARDS_IN_STOCK];
extern CardList *tableau[NUM_TABLEAU_SLOTS];
extern int remainingCards;
extern int finished_piles[8];
extern Undo *undo;

void checkForCompletedPiles(int slot);
void cleanup_game(void);
void deal_more_cards(void);
int game_get_completed_count(void);
CardList *getCardListAtPosition(int col, int row);
int get_suit_of_finished_pile(int pile);
void moveCards(int fromCol, int fromRow, int destCol, int bSaveUndo);
void reveal_top_card(int slot);
void game_shuffle(void);
void start_game(void);
void undo_last_move(void);

/* the "cardlist" functions */
CardList *cardlist_free(CardList *list);
int cardlist_len(const CardList *list);
CardList *cardlist_detach(CardList *list, CardList *part);

#endif /* GAME_H */
